import java.util.List;

/**
 * A Scribble player
 *
 * @author hajo
 */
// Pre: holder != null && bag != null && name != null && name.length() > 0
public class Player {

    private final Holder holder;
    private final String name;
    private final TileBag bag;  // Shared by all
    private int points;

    public Player(String name, Holder holder, TileBag bag) {
        if( name == null || holder == null || bag == null){
            throw new IllegalArgumentException("Null passed in");
        }
        if( name.length() == 0){
            throw new IllegalArgumentException("Name can't be empty");
        }
        this.name = name;
        this.bag = bag;
        this.holder = holder;
    }

    public void addPoints(int points) {
        this.points += points;
    }

    // Pre: 0 <= index < holder.size()
    public Tile pickTileFromHolder(int index) {
        return holder.remove(index);
    }

    // Pre: holder.actualSize() < holder.size() &&
    // bag.size() >= holder.size() - holder.actualSize()
    public void pickTilesFromBag() {
        // Will throw if null returned from bag
        while (holder.actualSize() < holder.size() && !bag.isEmpty()) {
            holder.add(bag.remove());
        }
    }

    public void putBackTile(Tile tile) {
        holder.add(tile);
    }

    public String getName() {
        return name;
    }

    public int getPoints() {
        return points;
    }

    @Override
    public String toString() {
        return "{" + name + ":" + holder.toString() + "}";
    }
}
